#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <pthread.h>

#include "headers.h"
#include "defines.h"

void site_locate_files(GTK_UI* my_ui) {
	SITE *site;
	FILE_ELEMENT* file_element;
	char *tmp;
	int res=0;
	
	site = my_ui->site;

	while(file_element = site_first_untested(site)) {
			printf("# %s\n", file_element->file_name);
			tmp = file_read(file_element->file_name, site->directory_root);
			string_scan(my_ui, tmp);
			string_to_filename_tree(	my_ui,
										tmp,
										file_element);
			free(tmp);
			site_move_to_file_found(my_ui, file_element);
	}
	printf("exit\n");
	pthread_exit(&res);
}

void site_add_file_name(	GTK_UI* my_ui,
							FILE_ELEMENT* file_element,
							char *scanned_fn) {
	char *s;
	SITE *site;
	FILE_ELEMENT *f, *dir_element;
	int i;
	gboolean nf, i_nf;
	gint is_dir;
	guint l;
	GString *myindex;
	GString *myext;

	site = my_ui->site;

	f = site_lookup(site, file_element->file_name);
	if(f == NULL) {
/*		printf("New   : %s\n",file_element->file_name);*/
		/* look for the extension */
		s = file_element->file_name+strlen(file_element->file_name);
		while((s[0] != '.') && (s > file_element->file_name)) s--;
		i = 0;
		nf = TRUE;
		while((i<(site->html_extensions)->len) && (nf == TRUE)) {
			myext = g_ptr_array_index(site->html_extensions,i);
			if(strcmp(s, myext->str) == 0) nf = FALSE;
			i++;
		}
		is_dir = file_test(file_element, site->directory_root);
		if(is_dir != FILE_DOES_NOT_EXIST) { /* test if file exists */
			if(nf == TRUE) { /* test if the extension is a html extension */
				if(is_dir == FILE_IS_DIRECTORY) { /* test if the file is a directory */
					i_nf = TRUE;
					l = (site->directory_indexes)->len;
					i = 0;
					dir_element = (FILE_ELEMENT*)malloc(sizeof(FILE_ELEMENT));
					while((i_nf == TRUE) && (i<l)) {
						myindex = g_ptr_array_index(site->directory_indexes,i);
						dir_element->file_name = (char*)malloc(sizeof(char)*(	strlen(file_element->file_name)
																+strlen(myindex->str)
																+2));
						strcpy(dir_element->file_name, file_element->file_name);
						if(dir_element->file_name[strlen(dir_element->file_name)-1] != '/')
							strcat(dir_element->file_name,"/");
						strcat(dir_element->file_name, myindex->str);
						
						if(file_test(dir_element, site->directory_root) != FILE_DOES_NOT_EXIST) {
							site_add_file_name(	my_ui, dir_element, scanned_fn);
							i_nf = FALSE;
						} else {
							free(dir_element->file_name);
						}
						i++;
					}
					if(i_nf == TRUE) free(dir_element);
				}
				file_element->status = FILE_FOUND;
				site_add_to_file_found(my_ui, file_element);
			} else {
				file_element->status = FILE_HTML_UNTESTED;
				site_add_to_file_to_test(my_ui, file_element);
			}
		} else {
			file_element->status = FILE_NOT_FOUND;
			site_add_to_file_not_found(my_ui, file_element, scanned_fn);
		}
	}
}

FILE_ELEMENT *filename_path_to_absolute(	SITE *site,
									char* new_fn,
									char* scanned_fn) {
	FILE_ELEMENT *returned_fe;
	char *s, *s2;
	char *w;
	int n;
	
	returned_fe = (FILE_ELEMENT*)malloc(sizeof(FILE_ELEMENT));
	
	if(new_fn[0] == '/') {
		returned_fe->file_name = (char*)malloc(sizeof(char)*(	strlen(new_fn)
																+strlen(site->directory_root)
																+1));		
		strcpy(returned_fe->file_name, site->directory_root);
		strcat(returned_fe->file_name, new_fn);
	} else {
		s = scanned_fn+strlen(scanned_fn)-1;
		while((s[0] != '/') && (s > scanned_fn)) s--;
		n = (s - scanned_fn) + 1;
		returned_fe->file_name = (char*)malloc(sizeof(char)*(	strlen(new_fn)
																	+n
																	+1));
		strncpy(returned_fe->file_name,scanned_fn, n);
		returned_fe->file_name[n] = 0;
		strcat(returned_fe->file_name, new_fn);
	}
	
	/* Look for loops like "../toto/../toto/" */
	s = returned_fe->file_name;
	while(strlen(s)>=3) {
		if(strncmp(s,"/../",4) == 0) {
			w = s-1;
			while((w[0] != '/') && (w > returned_fe->file_name)) w--;
			strcpy(w, s+3);
			s = w-1;
		}
		s++;
	}
	
	/* Look for loops like "/./././." */
	s = returned_fe->file_name;
	while(strlen(s)>=3) {
		if(strncmp(s,"/./",3) == 0) {
			strcpy(s, s+2);
			s--;
		}
		s++;
	}
	
	/* Look for "/////" in filenames */
	s = returned_fe->file_name;
	s2 = s;
	while(s[0]) {
		s2[0] = s[0];
		s++;
		if((s[0] != '/') || (s2[0] != '/')) s2++;
	}
	s2[0] = 0;
	
	
	return(returned_fe);
}

void string_to_filename_tree(	GTK_UI *my_ui,
								char *s,
								FILE_ELEMENT* scanned_file) {
	char *s1, *s2;
	SITE *site;
	FILE_ELEMENT *sc;
	char c;
	
	site = my_ui->site;
	
	/* s has the following syntax:
	 * "file1"file2"file3...filek"filem"filen
	 */
	s2 = s;
	/* for each file ...*/
	while(s2[0] == '"') {
		/* look for the end of this file and put a NULL char at the end */
		s2++;
		s1 = s2;
		while((s2[0]) && (s2[0] !='"')) s2++;
		c = s2[0];
		s2[0] = 0;
		
		/* just test if the file is local or on the net */
		if(	(g_strncasecmp(s1,"http:",5)) &&
			(g_strncasecmp(s1,"mailto:",7)) &&
			(g_strncasecmp(s1,"news:",5)) &&
			(g_strncasecmp(s1,"javascript:",11)) &&
			(g_strncasecmp(s1,"ftp:",4))) {
			
			/* Transform the filename to an absolute and local filename 
			 * This returns a FILE_ELEMENT*
			 */
			sc = filename_path_to_absolute(site, s1, scanned_file->file_name);
			
			/* Add the FILE_ELEMENT to the site tree and test that file */
			site_add_file_name(my_ui, sc, scanned_file->file_name);
		}
		s2[0] = c;
	}
}

