#include <stdio.h>
#include <string.h>
#include <glib.h>

#include "headers.h"
#include "defines.h"

SITE* site_create() {
	SITE* site;
	
	site = (SITE*)malloc(sizeof(SITE));
	site->file_not_found = NULL;
	site->file_found = NULL;
	site->file_to_test = NULL;
	site->files = g_hash_table_new(g_str_hash, g_str_equal);
	
	site->directory_root = NULL;
	return(site);
}

FILE_ELEMENT *site_lookup(SITE* site, char* key) {
	return((FILE_ELEMENT *)g_hash_table_lookup(site->files, key));
}

void site_add_to_file_not_found(GTK_UI* my_ui, FILE_ELEMENT *e, gchar *scanned_fn) {
	SITE *site;
	site = my_ui->site;
	
	e->list[0] = scanned_fn;
	e->list[1] = e->file_name;
	
	e->status = FILE_NOT_FOUND;
	site->file_not_found = g_list_append(site->file_not_found, e);
	g_hash_table_insert(site->files, e->file_name, e);
	
	gtk_clist_prepend(GTK_CLIST(my_ui->nbw_files_not_found), e->list);	
	
}

void site_add_to_file_found(GTK_UI* my_ui, FILE_ELEMENT *e) {
	SITE *site;
	site = my_ui->site;
	e->status = FILE_FOUND;
	site->file_found = g_list_append(site->file_found, e);
	g_hash_table_insert(site->files, e->file_name, e);
	gtk_clist_prepend(GTK_CLIST(my_ui->nbw_files_found), &(e->file_name));
	gtk_widget_draw_default(my_ui->nbsw_files_found);
	gtk_widget_draw_default(my_ui->nbw_files_found);
}

void site_add_to_file_to_test(GTK_UI* my_ui, FILE_ELEMENT *e) {
	SITE *site;
	site = my_ui->site;
	e->status = FILE_HTML_UNTESTED;
	site->file_to_test = g_list_append(site->file_to_test, e);
	g_hash_table_insert(site->files, e->file_name, e);
	gtk_clist_prepend(GTK_CLIST(my_ui->nbw_files_to_test), &(e->file_name));
}

void site_move_to_file_found(GTK_UI* my_ui, FILE_ELEMENT *e) {
	SITE *site;
	GList *l;
	gint i;
	gboolean nf;
	gchar **t;
	site = my_ui->site;
	
	e->status = FILE_FOUND;
	site->file_found = g_list_append(site->file_found, e);

	i = 0;
	nf = TRUE;
	while(nf == TRUE) {
		gtk_clist_get_text(GTK_CLIST(my_ui->nbw_files_to_test),i,0,t);
		if(strcmp(t[0],e->file_name) == 0) nf = FALSE;
		i++;
	}
	gtk_clist_remove(GTK_CLIST(my_ui->nbw_files_to_test),i-1);
	gtk_clist_prepend(GTK_CLIST(my_ui->nbw_files_found), &(e->file_name));
	gtk_widget_draw_default(my_ui->nbsw_files_found);
	gtk_widget_draw_default(my_ui->nbw_files_found);
	l = g_list_first(site->file_to_test);
	site->file_to_test = g_list_remove_link(site->file_to_test, l);
	g_list_free(l);
}

void site_free_fn(gpointer d, gpointer u) {
	free(((FILE_ELEMENT*)d)->file_name);
	free((FILE_ELEMENT*)d);
}

void site_free(SITE* site) {
	g_list_foreach(site->file_not_found, site_free_fn, NULL);
	g_list_foreach(site->file_found, site_free_fn, NULL);
	g_list_foreach(site->file_to_test, site_free_fn, NULL);
	g_list_free(site->file_not_found);
	g_list_free(site->file_found);
	g_list_free(site->file_to_test);
	g_hash_table_destroy(site->files);
	free(site);
}

FILE_ELEMENT* site_first_untested(SITE* site) {
	GList *g;
	g = g_list_first(site->file_to_test);
	printf("%d\n",g);
	if(g) return(g->data);
	else return(NULL);
}
